#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import imagingstudy
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class ImagingStudyTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("ImagingStudy", js["resourceType"])
        return imagingstudy.ImagingStudy(js)
    
    def testImagingStudy1(self):
        inst = self.instantiate_from("imagingstudy-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a ImagingStudy instance")
        self.implImagingStudy1(inst)
        
        js = inst.as_json()
        self.assertEqual("ImagingStudy", js["resourceType"])
        inst2 = imagingstudy.ImagingStudy(js)
        self.implImagingStudy1(inst2)
    
    def implImagingStudy1(self, inst):
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(inst.numberOfInstances, 1)
        self.assertEqual(inst.numberOfSeries, 1)
        self.assertEqual(force_bytes(inst.series[0].bodySite.code), force_bytes("67734004"))
        self.assertEqual(force_bytes(inst.series[0].bodySite.display), force_bytes("Upper Trunk Structure"))
        self.assertEqual(force_bytes(inst.series[0].bodySite.system), force_bytes("http://snomed.info/sct"))
        self.assertEqual(force_bytes(inst.series[0].description), force_bytes("CT Surview 180"))
        self.assertEqual(inst.series[0].instance[0].number, 1)
        self.assertEqual(force_bytes(inst.series[0].instance[0].sopClass), force_bytes("urn:oid:1.2.840.10008.5.1.4.1.1.2"))
        self.assertEqual(force_bytes(inst.series[0].instance[0].uid), force_bytes("urn:oid:2.16.124.113543.6003.189642796.63084.16748.2599092903"))
        self.assertEqual(force_bytes(inst.series[0].modality.code), force_bytes("CT"))
        self.assertEqual(force_bytes(inst.series[0].modality.system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(inst.series[0].number, 3)
        self.assertEqual(inst.series[0].numberOfInstances, 1)
        self.assertEqual(force_bytes(inst.series[0].uid), force_bytes("urn:oid:2.16.124.113543.6003.2588828330.45298.17418.2723805630"))
        self.assertEqual(inst.started.date, FHIRDate("2011-01-01T11:01:20+03:00").date)
        self.assertEqual(inst.started.as_json(), "2011-01-01T11:01:20+03:00")
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">CT Chest.  John Smith (MRN: 09236). Accession: W12342398. Performed: 2011-01-01. 3 series, 12 images.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.uid), force_bytes("urn:oid:2.16.124.113543.6003.1154777499.30246.19789.3503430045"))
    
    def testImagingStudy2(self):
        inst = self.instantiate_from("imagingstudy-example-xr.json")
        self.assertIsNotNone(inst, "Must have instantiated a ImagingStudy instance")
        self.implImagingStudy2(inst)
        
        js = inst.as_json()
        self.assertEqual("ImagingStudy", js["resourceType"])
        inst2 = imagingstudy.ImagingStudy(js)
        self.implImagingStudy2(inst2)
    
    def implImagingStudy2(self, inst):
        self.assertEqual(force_bytes(inst.accession.type.coding[0].code), force_bytes("ACSN"))
        self.assertEqual(force_bytes(inst.accession.type.coding[0].system), force_bytes("http://hl7.org/fhir/v2/0203"))
        self.assertEqual(force_bytes(inst.accession.use), force_bytes("usual"))
        self.assertEqual(force_bytes(inst.accession.value), force_bytes("W12342398"))
        self.assertEqual(force_bytes(inst.availability), force_bytes("ONLINE"))
        self.assertEqual(force_bytes(inst.description), force_bytes("XR Wrist 3+ Views"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example-xr"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("secondary"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("55551234"))
        self.assertEqual(force_bytes(inst.modalityList[0].code), force_bytes("DX"))
        self.assertEqual(force_bytes(inst.modalityList[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(inst.numberOfInstances, 2)
        self.assertEqual(inst.numberOfSeries, 1)
        self.assertEqual(force_bytes(inst.procedureCode[0].coding[0].code), force_bytes("RPID2589"))
        self.assertEqual(force_bytes(inst.procedureCode[0].coding[0].display), force_bytes("XR Wrist 3+ Views"))
        self.assertEqual(force_bytes(inst.procedureCode[0].coding[0].system), force_bytes("http://www.radlex.org"))
        self.assertEqual(force_bytes(inst.procedureCode[0].text), force_bytes("XR Wrist 3+ Views"))
        self.assertEqual(force_bytes(inst.reason.coding[0].code), force_bytes("357009"))
        self.assertEqual(force_bytes(inst.reason.coding[0].display), force_bytes("Closed fracture of trapezoidal bone of wrist"))
        self.assertEqual(force_bytes(inst.reason.coding[0].system), force_bytes("http://snomed.info/sct"))
        self.assertEqual(force_bytes(inst.series[0].availability), force_bytes("ONLINE"))
        self.assertEqual(force_bytes(inst.series[0].bodySite.code), force_bytes("T-15460"))
        self.assertEqual(force_bytes(inst.series[0].bodySite.display), force_bytes("Wrist Joint"))
        self.assertEqual(force_bytes(inst.series[0].bodySite.system), force_bytes("http://snomed.info/sct"))
        self.assertEqual(force_bytes(inst.series[0].description), force_bytes("XR Wrist 3+ Views"))
        self.assertEqual(inst.series[0].instance[0].number, 1)
        self.assertEqual(force_bytes(inst.series[0].instance[0].sopClass), force_bytes("urn:oid:1.2.840.10008.5.1.4.1.1.2"))
        self.assertEqual(force_bytes(inst.series[0].instance[0].title), force_bytes("PA VIEW"))
        self.assertEqual(force_bytes(inst.series[0].instance[0].uid), force_bytes("urn:oid:2.16.124.113543.6003.1154777499.30246.19789.3503430045.1.1"))
        self.assertEqual(inst.series[0].instance[1].number, 2)
        self.assertEqual(force_bytes(inst.series[0].instance[1].sopClass), force_bytes("urn:oid:1.2.840.10008.5.1.4.1.1.2"))
        self.assertEqual(force_bytes(inst.series[0].instance[1].title), force_bytes("LL VIEW"))
        self.assertEqual(force_bytes(inst.series[0].instance[1].uid), force_bytes("urn:oid:2.16.124.113543.6003.1154777499.30246.19789.3503430045.1.2"))
        self.assertEqual(force_bytes(inst.series[0].laterality.code), force_bytes("419161000"))
        self.assertEqual(force_bytes(inst.series[0].laterality.display), force_bytes("Unilateral left"))
        self.assertEqual(force_bytes(inst.series[0].laterality.system), force_bytes("http://snomed.info/sct"))
        self.assertEqual(force_bytes(inst.series[0].modality.code), force_bytes("DX"))
        self.assertEqual(force_bytes(inst.series[0].modality.system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(inst.series[0].number, 3)
        self.assertEqual(inst.series[0].numberOfInstances, 2)
        self.assertEqual(inst.series[0].started.date, FHIRDate("2011-01-01T11:01:20+03:00").date)
        self.assertEqual(inst.series[0].started.as_json(), "2011-01-01T11:01:20+03:00")
        self.assertEqual(force_bytes(inst.series[0].uid), force_bytes("urn:oid:2.16.124.113543.6003.1154777499.30246.19789.3503430045.1"))
        self.assertEqual(inst.started.date, FHIRDate("2017-01-01T11:01:20+03:00").date)
        self.assertEqual(inst.started.as_json(), "2017-01-01T11:01:20+03:00")
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">XR Wrist 3+ Views. John Smith (MRN: 09236). Accession: W12342398. Performed: 2017-01-01. 1 series, 2 images.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.uid), force_bytes("urn:oid:2.16.124.113543.6003.1154777499.30246.19789.3503430046"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import chargeitem
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class ChargeItemTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("ChargeItem", js["resourceType"])
        return chargeitem.ChargeItem(js)
    
    def testChargeItem1(self):
        inst = self.instantiate_from("chargeitem-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a ChargeItem instance")
        self.implChargeItem1(inst)
        
        js = inst.as_json()
        self.assertEqual("ChargeItem", js["resourceType"])
        inst2 = chargeitem.ChargeItem(js)
        self.implChargeItem1(inst2)
    
    def implChargeItem1(self, inst):
        self.assertEqual(force_bytes(inst.code.coding[0].code), force_bytes("01510"))
        self.assertEqual(force_bytes(inst.code.coding[0].display), force_bytes("Zusatzpauschale für Beobachtung nach diagnostischer Koronarangiografie"))
        self.assertEqual(force_bytes(inst.definition[0]), force_bytes("http://www.kbv.de/tools/ebm/html/01520_2904360860826220813632.html"))
        self.assertEqual(inst.enteredDate.date, FHIRDate("2017-01-25T23:55:04+01:00").date)
        self.assertEqual(inst.enteredDate.as_json(), "2017-01-25T23:55:04+01:00")
        self.assertEqual(inst.factorOverride, 0.8)
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(force_bytes(inst.identifier.system), force_bytes("http://myHospital.org/ChargeItems"))
        self.assertEqual(force_bytes(inst.identifier.value), force_bytes("654321"))
        self.assertEqual(force_bytes(inst.note[0].text), force_bytes("The code is only applicable for periods longer than 4h"))
        self.assertEqual(inst.note[0].time.date, FHIRDate("2017-01-25T23:55:04+01:00").date)
        self.assertEqual(inst.note[0].time.as_json(), "2017-01-25T23:55:04+01:00")
        self.assertEqual(inst.occurrencePeriod.end.date, FHIRDate("2017-01-25T12:35:00+01:00").date)
        self.assertEqual(inst.occurrencePeriod.end.as_json(), "2017-01-25T12:35:00+01:00")
        self.assertEqual(inst.occurrencePeriod.start.date, FHIRDate("2017-01-25T08:00:00+01:00").date)
        self.assertEqual(inst.occurrencePeriod.start.as_json(), "2017-01-25T08:00:00+01:00")
        self.assertEqual(force_bytes(inst.overrideReason), force_bytes("Patient is Cardiologist's golf buddy, so he gets a 20% discount!"))
        self.assertEqual(force_bytes(inst.participant[0].role.coding[0].code), force_bytes("17561000"))
        self.assertEqual(force_bytes(inst.participant[0].role.coding[0].display), force_bytes("Cardiologist"))
        self.assertEqual(force_bytes(inst.participant[0].role.coding[0].system), force_bytes("http://snomed.info/sct"))
        self.assertEqual(force_bytes(inst.participant[1].role.coding[0].code), force_bytes("224542009"))
        self.assertEqual(force_bytes(inst.participant[1].role.coding[0].display), force_bytes("Coronary Care Nurse"))
        self.assertEqual(force_bytes(inst.participant[1].role.coding[0].system), force_bytes("http://snomed.info/sct"))
        self.assertEqual(force_bytes(inst.priceOverride.code), force_bytes("EUR"))
        self.assertEqual(force_bytes(inst.priceOverride.system), force_bytes("urn:iso:std:iso:4217"))
        self.assertEqual(force_bytes(inst.priceOverride.unit), force_bytes("EUR"))
        self.assertEqual(inst.priceOverride.value, 40)
        self.assertEqual(inst.quantity.value, 1)
        self.assertEqual(force_bytes(inst.reason[0].coding[0].code), force_bytes("123456"))
        self.assertEqual(force_bytes(inst.reason[0].coding[0].display), force_bytes("DIAG-1"))
        self.assertEqual(force_bytes(inst.reason[0].coding[0].system), force_bytes("http://hl7.org/fhir/sid/icd-10"))
        self.assertEqual(force_bytes(inst.status), force_bytes("billable"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">Example of ChargeItem Usage in Context of the German EBM Billing code system</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/CapabilityStatement) on 2019-01-12.
#  2019, SMART Health IT.


from . import domainresource

class CapabilityStatement(domainresource.DomainResource):
    """ A statement of system capabilities.
    
    A Capability Statement documents a set of capabilities (behaviors) of a
    FHIR Server that may be used as a statement of actual server functionality
    or a statement of required or desired server implementation.
    """
    
    resource_type = "CapabilityStatement"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.acceptUnknown = None
        """ no | extensions | elements | both.
        Type `str`. """
        
        self.contact = None
        """ Contact details for the publisher.
        List of `ContactDetail` items (represented as `dict` in JSON). """
        
        self.copyright = None
        """ Use and/or publishing restrictions.
        Type `str`. """
        
        self.date = None
        """ Date this was last changed.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.description = None
        """ Natural language description of the capability statement.
        Type `str`. """
        
        self.document = None
        """ Document definition.
        List of `CapabilityStatementDocument` items (represented as `dict` in JSON). """
        
        self.experimental = None
        """ For testing purposes, not real usage.
        Type `bool`. """
        
        self.fhirVersion = None
        """ FHIR Version the system uses.
        Type `str`. """
        
        self.format = None
        """ formats supported (xml | json | ttl | mime type).
        List of `str` items. """
        
        self.implementation = None
        """ If this describes a specific instance.
        Type `CapabilityStatementImplementation` (represented as `dict` in JSON). """
        
        self.implementationGuide = None
        """ Implementation guides supported.
        List of `str` items. """
        
        self.instantiates = None
        """ Canonical URL of another capability statement this implements.
        List of `str` items. """
        
        self.jurisdiction = None
        """ Intended jurisdiction for capability statement (if applicable).
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.kind = None
        """ instance | capability | requirements.
        Type `str`. """
        
        self.messaging = None
        """ If messaging is supported.
        List of `CapabilityStatementMessaging` items (represented as `dict` in JSON). """
        
        self.name = None
        """ Name for this capability statement (computer friendly).
        Type `str`. """
        
        self.patchFormat = None
        """ Patch formats supported.
        List of `str` items. """
        
        self.profile = None
        """ Profiles for use cases supported.
        List of `FHIRReference` items referencing `StructureDefinition` (represented as `dict` in JSON). """
        
        self.publisher = None
        """ Name of the publisher (organization or individual).
        Type `str`. """
        
        self.purpose = None
        """ Why this capability statement is defined.
        Type `str`. """
        
        self.rest = None
        """ If the endpoint is a RESTful one.
        List of `CapabilityStatementRest` items (represented as `dict` in JSON). """
        
        self.software = None
        """ Software that is covered by this capability statement.
        Type `CapabilityStatementSoftware` (represented as `dict` in JSON). """
        
        self.status = None
        """ draft | active | retired | unknown.
        Type `str`. """
        
        self.title = None
        """ Name for this capability statement (human friendly).
        Type `str`. """
        
        self.url = None
        """ Logical URI to reference this capability statement (globally
        unique).
        Type `str`. """
        
        self.useContext = None
        """ Context the content is intended to support.
        List of `UsageContext` items (represented as `dict` in JSON). """
        
        self.version = None
        """ Business version of the capability statement.
        Type `str`. """
        
        super(CapabilityStatement, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatement, self).elementProperties()
        js.extend([
            ("acceptUnknown", "acceptUnknown", str, False, None, True),
            ("contact", "contact", contactdetail.ContactDetail, True, None, False),
            ("copyright", "copyright", str, False, None, False),
            ("date", "date", fhirdate.FHIRDate, False, None, True),
            ("description", "description", str, False, None, False),
            ("document", "document", CapabilityStatementDocument, True, None, False),
            ("experimental", "experimental", bool, False, None, False),
            ("fhirVersion", "fhirVersion", str, False, None, True),
            ("format", "format", str, True, None, True),
            ("implementation", "implementation", CapabilityStatementImplementation, False, None, False),
            ("implementationGuide", "implementationGuide", str, True, None, False),
            ("instantiates", "instantiates", str, True, None, False),
            ("jurisdiction", "jurisdiction", codeableconcept.CodeableConcept, True, None, False),
            ("kind", "kind", str, False, None, True),
            ("messaging", "messaging", CapabilityStatementMessaging, True, None, False),
            ("name", "name", str, False, None, False),
            ("patchFormat", "patchFormat", str, True, None, False),
            ("profile", "profile", fhirreference.FHIRReference, True, None, False),
            ("publisher", "publisher", str, False, None, False),
            ("purpose", "purpose", str, False, None, False),
            ("rest", "rest", CapabilityStatementRest, True, None, False),
            ("software", "software", CapabilityStatementSoftware, False, None, False),
            ("status", "status", str, False, None, True),
            ("title", "title", str, False, None, False),
            ("url", "url", str, False, None, False),
            ("useContext", "useContext", usagecontext.UsageContext, True, None, False),
            ("version", "version", str, False, None, False),
        ])
        return js


from . import backboneelement

class CapabilityStatementDocument(backboneelement.BackboneElement):
    """ Document definition.
    
    A document definition.
    """
    
    resource_type = "CapabilityStatementDocument"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.documentation = None
        """ Description of document support.
        Type `str`. """
        
        self.mode = None
        """ producer | consumer.
        Type `str`. """
        
        self.profile = None
        """ Constraint on a resource used in the document.
        Type `FHIRReference` referencing `StructureDefinition` (represented as `dict` in JSON). """
        
        super(CapabilityStatementDocument, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementDocument, self).elementProperties()
        js.extend([
            ("documentation", "documentation", str, False, None, False),
            ("mode", "mode", str, False, None, True),
            ("profile", "profile", fhirreference.FHIRReference, False, None, True),
        ])
        return js


class CapabilityStatementImplementation(backboneelement.BackboneElement):
    """ If this describes a specific instance.
    
    Identifies a specific implementation instance that is described by the
    capability statement - i.e. a particular installation, rather than the
    capabilities of a software program.
    """
    
    resource_type = "CapabilityStatementImplementation"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.description = None
        """ Describes this specific instance.
        Type `str`. """
        
        self.url = None
        """ Base URL for the installation.
        Type `str`. """
        
        super(CapabilityStatementImplementation, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementImplementation, self).elementProperties()
        js.extend([
            ("description", "description", str, False, None, True),
            ("url", "url", str, False, None, False),
        ])
        return js


class CapabilityStatementMessaging(backboneelement.BackboneElement):
    """ If messaging is supported.
    
    A description of the messaging capabilities of the solution.
    """
    
    resource_type = "CapabilityStatementMessaging"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.documentation = None
        """ Messaging interface behavior details.
        Type `str`. """
        
        self.endpoint = None
        """ Where messages should be sent.
        List of `CapabilityStatementMessagingEndpoint` items (represented as `dict` in JSON). """
        
        self.event = None
        """ Declare support for this event.
        List of `CapabilityStatementMessagingEvent` items (represented as `dict` in JSON). """
        
        self.reliableCache = None
        """ Reliable Message Cache Length (min).
        Type `int`. """
        
        self.supportedMessage = None
        """ Messages supported by this system.
        List of `CapabilityStatementMessagingSupportedMessage` items (represented as `dict` in JSON). """
        
        super(CapabilityStatementMessaging, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementMessaging, self).elementProperties()
        js.extend([
            ("documentation", "documentation", str, False, None, False),
            ("endpoint", "endpoint", CapabilityStatementMessagingEndpoint, True, None, False),
            ("event", "event", CapabilityStatementMessagingEvent, True, None, False),
            ("reliableCache", "reliableCache", int, False, None, False),
            ("supportedMessage", "supportedMessage", CapabilityStatementMessagingSupportedMessage, True, None, False),
        ])
        return js


class CapabilityStatementMessagingEndpoint(backboneelement.BackboneElement):
    """ Where messages should be sent.
    
    An endpoint (network accessible address) to which messages and/or replies
    are to be sent.
    """
    
    resource_type = "CapabilityStatementMessagingEndpoint"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.address = None
        """ Network address or identifier of the end-point.
        Type `str`. """
        
        self.protocol = None
        """ http | ftp | mllp +.
        Type `Coding` (represented as `dict` in JSON). """
        
        super(CapabilityStatementMessagingEndpoint, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementMessagingEndpoint, self).elementProperties()
        js.extend([
            ("address", "address", str, False, None, True),
            ("protocol", "protocol", coding.Coding, False, None, True),
        ])
        return js


class CapabilityStatementMessagingEvent(backboneelement.BackboneElement):
    """ Declare support for this event.
    
    A description of the solution's support for an event at this end-point.
    """
    
    resource_type = "CapabilityStatementMessagingEvent"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.category = None
        """ Consequence | Currency | Notification.
        Type `str`. """
        
        self.code = None
        """ Event type.
        Type `Coding` (represented as `dict` in JSON). """
        
        self.documentation = None
        """ Endpoint-specific event documentation.
        Type `str`. """
        
        self.focus = None
        """ Resource that's focus of message.
        Type `str`. """
        
        self.mode = None
        """ sender | receiver.
        Type `str`. """
        
        self.request = None
        """ Profile that describes the request.
        Type `FHIRReference` referencing `StructureDefinition` (represented as `dict` in JSON). """
        
        self.response = None
        """ Profile that describes the response.
        Type `FHIRReference` referencing `StructureDefinition` (represented as `dict` in JSON). """
        
        super(CapabilityStatementMessagingEvent, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementMessagingEvent, self).elementProperties()
        js.extend([
            ("category", "category", str, False, None, False),
            ("code", "code", coding.Coding, False, None, True),
            ("documentation", "documentation", str, False, None, False),
            ("focus", "focus", str, False, None, True),
            ("mode", "mode", str, False, None, True),
            ("request", "request", fhirreference.FHIRReference, False, None, True),
            ("response", "response", fhirreference.FHIRReference, False, None, True),
        ])
        return js


class CapabilityStatementMessagingSupportedMessage(backboneelement.BackboneElement):
    """ Messages supported by this system.
    
    References to message definitions for messages this system can send or
    receive.
    """
    
    resource_type = "CapabilityStatementMessagingSupportedMessage"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.definition = None
        """ Message supported by this system.
        Type `FHIRReference` referencing `MessageDefinition` (represented as `dict` in JSON). """
        
        self.mode = None
        """ sender | receiver.
        Type `str`. """
        
        super(CapabilityStatementMessagingSupportedMessage, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementMessagingSupportedMessage, self).elementProperties()
        js.extend([
            ("definition", "definition", fhirreference.FHIRReference, False, None, True),
            ("mode", "mode", str, False, None, True),
        ])
        return js


class CapabilityStatementRest(backboneelement.BackboneElement):
    """ If the endpoint is a RESTful one.
    
    A definition of the restful capabilities of the solution, if any.
    """
    
    resource_type = "CapabilityStatementRest"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.compartment = None
        """ Compartments served/used by system.
        List of `str` items. """
        
        self.documentation = None
        """ General description of implementation.
        Type `str`. """
        
        self.interaction = None
        """ What operations are supported?.
        List of `CapabilityStatementRestInteraction` items (represented as `dict` in JSON). """
        
        self.mode = None
        """ client | server.
        Type `str`. """
        
        self.operation = None
        """ Definition of an operation or a custom query.
        List of `CapabilityStatementRestOperation` items (represented as `dict` in JSON). """
        
        self.resource = None
        """ Resource served on the REST interface.
        List of `CapabilityStatementRestResource` items (represented as `dict` in JSON). """
        
        self.searchParam = None
        """ Search parameters for searching all resources.
        List of `CapabilityStatementRestResourceSearchParam` items (represented as `dict` in JSON). """
        
        self.security = None
        """ Information about security of implementation.
        Type `CapabilityStatementRestSecurity` (represented as `dict` in JSON). """
        
        super(CapabilityStatementRest, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementRest, self).elementProperties()
        js.extend([
            ("compartment", "compartment", str, True, None, False),
            ("documentation", "documentation", str, False, None, False),
            ("interaction", "interaction", CapabilityStatementRestInteraction, True, None, False),
            ("mode", "mode", str, False, None, True),
            ("operation", "operation", CapabilityStatementRestOperation, True, None, False),
            ("resource", "resource", CapabilityStatementRestResource, True, None, False),
            ("searchParam", "searchParam", CapabilityStatementRestResourceSearchParam, True, None, False),
            ("security", "security", CapabilityStatementRestSecurity, False, None, False),
        ])
        return js


class CapabilityStatementRestInteraction(backboneelement.BackboneElement):
    """ What operations are supported?.
    
    A specification of restful operations supported by the system.
    """
    
    resource_type = "CapabilityStatementRestInteraction"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.code = None
        """ transaction | batch | search-system | history-system.
        Type `str`. """
        
        self.documentation = None
        """ Anything special about operation behavior.
        Type `str`. """
        
        super(CapabilityStatementRestInteraction, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementRestInteraction, self).elementProperties()
        js.extend([
            ("code", "code", str, False, None, True),
            ("documentation", "documentation", str, False, None, False),
        ])
        return js


class CapabilityStatementRestOperation(backboneelement.BackboneElement):
    """ Definition of an operation or a custom query.
    
    Definition of an operation or a named query together with its parameters
    and their meaning and type.
    """
    
    resource_type = "CapabilityStatementRestOperation"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.definition = None
        """ The defined operation/query.
        Type `FHIRReference` referencing `OperationDefinition` (represented as `dict` in JSON). """
        
        self.name = None
        """ Name by which the operation/query is invoked.
        Type `str`. """
        
        super(CapabilityStatementRestOperation, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementRestOperation, self).elementProperties()
        js.extend([
            ("definition", "definition", fhirreference.FHIRReference, False, None, True),
            ("name", "name", str, False, None, True),
        ])
        return js


class CapabilityStatementRestResource(backboneelement.BackboneElement):
    """ Resource served on the REST interface.
    
    A specification of the restful capabilities of the solution for a specific
    resource type.
    """
    
    resource_type = "CapabilityStatementRestResource"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.conditionalCreate = None
        """ If allows/uses conditional create.
        Type `bool`. """
        
        self.conditionalDelete = None
        """ not-supported | single | multiple - how conditional delete is
        supported.
        Type `str`. """
        
        self.conditionalRead = None
        """ not-supported | modified-since | not-match | full-support.
        Type `str`. """
        
        self.conditionalUpdate = None
        """ If allows/uses conditional update.
        Type `bool`. """
        
        self.documentation = None
        """ Additional information about the use of the resource type.
        Type `str`. """
        
        self.interaction = None
        """ What operations are supported?.
        List of `CapabilityStatementRestResourceInteraction` items (represented as `dict` in JSON). """
        
        self.profile = None
        """ Base System profile for all uses of resource.
        Type `FHIRReference` referencing `StructureDefinition` (represented as `dict` in JSON). """
        
        self.readHistory = None
        """ Whether vRead can return past versions.
        Type `bool`. """
        
        self.referencePolicy = None
        """ literal | logical | resolves | enforced | local.
        List of `str` items. """
        
        self.searchInclude = None
        """ _include values supported by the server.
        List of `str` items. """
        
        self.searchParam = None
        """ Search parameters supported by implementation.
        List of `CapabilityStatementRestResourceSearchParam` items (represented as `dict` in JSON). """
        
        self.searchRevInclude = None
        """ _revinclude values supported by the server.
        List of `str` items. """
        
        self.type = None
        """ A resource type that is supported.
        Type `str`. """
        
        self.updateCreate = None
        """ If update can commit to a new identity.
        Type `bool`. """
        
        self.versioning = None
        """ no-version | versioned | versioned-update.
        Type `str`. """
        
        super(CapabilityStatementRestResource, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementRestResource, self).elementProperties()
        js.extend([
            ("conditionalCreate", "conditionalCreate", bool, False, None, False),
            ("conditionalDelete", "conditionalDelete", str, False, None, False),
            ("conditionalRead", "conditionalRead", str, False, None, False),
            ("conditionalUpdate", "conditionalUpdate", bool, False, None, False),
            ("documentation", "documentation", str, False, None, False),
            ("interaction", "interaction", CapabilityStatementRestResourceInteraction, True, None, True),
            ("profile", "profile", fhirreference.FHIRReference, False, None, False),
            ("readHistory", "readHistory", bool, False, None, False),
            ("referencePolicy", "referencePolicy", str, True, None, False),
            ("searchInclude", "searchInclude", str, True, None, False),
            ("searchParam", "searchParam", CapabilityStatementRestResourceSearchParam, True, None, False),
            ("searchRevInclude", "searchRevInclude", str, True, None, False),
            ("type", "type", str, False, None, True),
            ("updateCreate", "updateCreate", bool, False, None, False),
            ("versioning", "versioning", str, False, None, False),
        ])
        return js


class CapabilityStatementRestResourceInteraction(backboneelement.BackboneElement):
    """ What operations are supported?.
    
    Identifies a restful operation supported by the solution.
    """
    
    resource_type = "CapabilityStatementRestResourceInteraction"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.code = None
        """ read | vread | update | patch | delete | history-instance |
        history-type | create | search-type.
        Type `str`. """
        
        self.documentation = None
        """ Anything special about operation behavior.
        Type `str`. """
        
        super(CapabilityStatementRestResourceInteraction, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementRestResourceInteraction, self).elementProperties()
        js.extend([
            ("code", "code", str, False, None, True),
            ("documentation", "documentation", str, False, None, False),
        ])
        return js


class CapabilityStatementRestResourceSearchParam(backboneelement.BackboneElement):
    """ Search parameters supported by implementation.
    
    Search parameters for implementations to support and/or make use of -
    either references to ones defined in the specification, or additional ones
    defined for/by the implementation.
    """
    
    resource_type = "CapabilityStatementRestResourceSearchParam"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.definition = None
        """ Source of definition for parameter.
        Type `str`. """
        
        self.documentation = None
        """ Server-specific usage.
        Type `str`. """
        
        self.name = None
        """ Name of search parameter.
        Type `str`. """
        
        self.type = None
        """ number | date | string | token | reference | composite | quantity |
        uri.
        Type `str`. """
        
        super(CapabilityStatementRestResourceSearchParam, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementRestResourceSearchParam, self).elementProperties()
        js.extend([
            ("definition", "definition", str, False, None, False),
            ("documentation", "documentation", str, False, None, False),
            ("name", "name", str, False, None, True),
            ("type", "type", str, False, None, True),
        ])
        return js


class CapabilityStatementRestSecurity(backboneelement.BackboneElement):
    """ Information about security of implementation.
    
    Information about security implementation from an interface perspective -
    what a client needs to know.
    """
    
    resource_type = "CapabilityStatementRestSecurity"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.certificate = None
        """ Certificates associated with security profiles.
        List of `CapabilityStatementRestSecurityCertificate` items (represented as `dict` in JSON). """
        
        self.cors = None
        """ Adds CORS Headers (http://enable-cors.org/).
        Type `bool`. """
        
        self.description = None
        """ General description of how security works.
        Type `str`. """
        
        self.service = None
        """ OAuth | SMART-on-FHIR | NTLM | Basic | Kerberos | Certificates.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        super(CapabilityStatementRestSecurity, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementRestSecurity, self).elementProperties()
        js.extend([
            ("certificate", "certificate", CapabilityStatementRestSecurityCertificate, True, None, False),
            ("cors", "cors", bool, False, None, False),
            ("description", "description", str, False, None, False),
            ("service", "service", codeableconcept.CodeableConcept, True, None, False),
        ])
        return js


class CapabilityStatementRestSecurityCertificate(backboneelement.BackboneElement):
    """ Certificates associated with security profiles.
    """
    
    resource_type = "CapabilityStatementRestSecurityCertificate"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.blob = None
        """ Actual certificate.
        Type `str`. """
        
        self.type = None
        """ Mime type for certificates.
        Type `str`. """
        
        super(CapabilityStatementRestSecurityCertificate, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementRestSecurityCertificate, self).elementProperties()
        js.extend([
            ("blob", "blob", str, False, None, False),
            ("type", "type", str, False, None, False),
        ])
        return js


class CapabilityStatementSoftware(backboneelement.BackboneElement):
    """ Software that is covered by this capability statement.
    
    Software that is covered by this capability statement.  It is used when the
    capability statement describes the capabilities of a particular software
    version, independent of an installation.
    """
    
    resource_type = "CapabilityStatementSoftware"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.name = None
        """ A name the software is known by.
        Type `str`. """
        
        self.releaseDate = None
        """ Date this version released.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.version = None
        """ Version covered by this statement.
        Type `str`. """
        
        super(CapabilityStatementSoftware, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CapabilityStatementSoftware, self).elementProperties()
        js.extend([
            ("name", "name", str, False, None, True),
            ("releaseDate", "releaseDate", fhirdate.FHIRDate, False, None, False),
            ("version", "version", str, False, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import coding
except ImportError:
    coding = sys.modules[__package__ + '.coding']
try:
    from . import contactdetail
except ImportError:
    contactdetail = sys.modules[__package__ + '.contactdetail']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import usagecontext
except ImportError:
    usagecontext = sys.modules[__package__ + '.usagecontext']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import auditevent
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class AuditEventTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("AuditEvent", js["resourceType"])
        return auditevent.AuditEvent(js)
    
    def testAuditEvent1(self):
        inst = self.instantiate_from("audit-event-example-vread.json")
        self.assertIsNotNone(inst, "Must have instantiated a AuditEvent instance")
        self.implAuditEvent1(inst)
        
        js = inst.as_json()
        self.assertEqual("AuditEvent", js["resourceType"])
        inst2 = auditevent.AuditEvent(js)
        self.implAuditEvent1(inst2)
    
    def implAuditEvent1(self, inst):
        self.assertEqual(force_bytes(inst.action), force_bytes("R"))
        self.assertEqual(force_bytes(inst.agent[0].altId), force_bytes("601847123"))
        self.assertEqual(force_bytes(inst.agent[0].name), force_bytes("Grahame Grieve"))
        self.assertTrue(inst.agent[0].requestor)
        self.assertEqual(force_bytes(inst.agent[0].userId.value), force_bytes("95"))
        self.assertEqual(force_bytes(inst.agent[1].altId), force_bytes("6580"))
        self.assertEqual(force_bytes(inst.agent[1].network.address), force_bytes("Workstation1.ehr.familyclinic.com"))
        self.assertEqual(force_bytes(inst.agent[1].network.type), force_bytes("1"))
        self.assertFalse(inst.agent[1].requestor)
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].code), force_bytes("110153"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].display), force_bytes("Source Role ID"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[1].userId.value), force_bytes("2.16.840.1.113883.4.2|2.16.840.1.113883.4.2"))
        self.assertEqual(force_bytes(inst.entity[0].lifecycle.code), force_bytes("6"))
        self.assertEqual(force_bytes(inst.entity[0].lifecycle.display), force_bytes("Access / Use"))
        self.assertEqual(force_bytes(inst.entity[0].lifecycle.system), force_bytes("http://hl7.org/fhir/dicom-audit-lifecycle"))
        self.assertEqual(force_bytes(inst.entity[0].type.code), force_bytes("2"))
        self.assertEqual(force_bytes(inst.entity[0].type.display), force_bytes("System Object"))
        self.assertEqual(force_bytes(inst.entity[0].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example-rest"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("0"))
        self.assertEqual(inst.recorded.date, FHIRDate("2013-06-20T23:42:24Z").date)
        self.assertEqual(inst.recorded.as_json(), "2013-06-20T23:42:24Z")
        self.assertEqual(force_bytes(inst.source.identifier.value), force_bytes("hl7connect.healthintersections.com.au"))
        self.assertEqual(force_bytes(inst.source.site), force_bytes("Cloud"))
        self.assertEqual(force_bytes(inst.source.type[0].code), force_bytes("3"))
        self.assertEqual(force_bytes(inst.source.type[0].display), force_bytes("Web Server"))
        self.assertEqual(force_bytes(inst.source.type[0].system), force_bytes("http://hl7.org/fhir/security-source-type"))
        self.assertEqual(force_bytes(inst.subtype[0].code), force_bytes("vread"))
        self.assertEqual(force_bytes(inst.subtype[0].display), force_bytes("vread"))
        self.assertEqual(force_bytes(inst.subtype[0].system), force_bytes("http://hl7.org/fhir/restful-interaction"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.code), force_bytes("rest"))
        self.assertEqual(force_bytes(inst.type.display), force_bytes("Restful Operation"))
        self.assertEqual(force_bytes(inst.type.system), force_bytes("http://hl7.org/fhir/audit-event-type"))
    
    def testAuditEvent2(self):
        inst = self.instantiate_from("auditevent-example-disclosure.json")
        self.assertIsNotNone(inst, "Must have instantiated a AuditEvent instance")
        self.implAuditEvent2(inst)
        
        js = inst.as_json()
        self.assertEqual("AuditEvent", js["resourceType"])
        inst2 = auditevent.AuditEvent(js)
        self.implAuditEvent2(inst2)
    
    def implAuditEvent2(self, inst):
        self.assertEqual(force_bytes(inst.action), force_bytes("R"))
        self.assertEqual(force_bytes(inst.agent[0].altId), force_bytes("notMe"))
        self.assertEqual(force_bytes(inst.agent[0].name), force_bytes("That guy everyone wishes would be caught"))
        self.assertEqual(force_bytes(inst.agent[0].network.address), force_bytes("custodian.net"))
        self.assertEqual(force_bytes(inst.agent[0].network.type), force_bytes("1"))
        self.assertEqual(force_bytes(inst.agent[0].policy[0]), force_bytes("http://consent.com/yes"))
        self.assertTrue(inst.agent[0].requestor)
        self.assertEqual(force_bytes(inst.agent[0].role[0].coding[0].code), force_bytes("110153"))
        self.assertEqual(force_bytes(inst.agent[0].role[0].coding[0].display), force_bytes("Source Role ID"))
        self.assertEqual(force_bytes(inst.agent[0].role[0].coding[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[0].userId.value), force_bytes("SomeIdiot@nowhere.com"))
        self.assertEqual(force_bytes(inst.agent[1].network.address), force_bytes("marketing.land"))
        self.assertEqual(force_bytes(inst.agent[1].network.type), force_bytes("1"))
        self.assertEqual(force_bytes(inst.agent[1].purposeOfUse[0].coding[0].code), force_bytes("HMARKT"))
        self.assertEqual(force_bytes(inst.agent[1].purposeOfUse[0].coding[0].display), force_bytes("healthcare marketing"))
        self.assertEqual(force_bytes(inst.agent[1].purposeOfUse[0].coding[0].system), force_bytes("http://hl7.org/fhir/v3/ActReason"))
        self.assertFalse(inst.agent[1].requestor)
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].code), force_bytes("110152"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].display), force_bytes("Destination Role ID"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[1].userId.value), force_bytes("Where"))
        self.assertEqual(force_bytes(inst.entity[0].role.code), force_bytes("1"))
        self.assertEqual(force_bytes(inst.entity[0].role.display), force_bytes("Patient"))
        self.assertEqual(force_bytes(inst.entity[0].role.system), force_bytes("http://hl7.org/fhir/object-role"))
        self.assertEqual(force_bytes(inst.entity[0].type.code), force_bytes("1"))
        self.assertEqual(force_bytes(inst.entity[0].type.display), force_bytes("Person"))
        self.assertEqual(force_bytes(inst.entity[0].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.entity[1].description), force_bytes("data about Everthing important"))
        self.assertEqual(force_bytes(inst.entity[1].identifier.value), force_bytes("What.id"))
        self.assertEqual(force_bytes(inst.entity[1].lifecycle.code), force_bytes("11"))
        self.assertEqual(force_bytes(inst.entity[1].lifecycle.display), force_bytes("Disclosure"))
        self.assertEqual(force_bytes(inst.entity[1].lifecycle.system), force_bytes("http://hl7.org/fhir/dicom-audit-lifecycle"))
        self.assertEqual(force_bytes(inst.entity[1].name), force_bytes("Namne of What"))
        self.assertEqual(force_bytes(inst.entity[1].role.code), force_bytes("4"))
        self.assertEqual(force_bytes(inst.entity[1].role.display), force_bytes("Domain Resource"))
        self.assertEqual(force_bytes(inst.entity[1].role.system), force_bytes("http://hl7.org/fhir/object-role"))
        self.assertEqual(force_bytes(inst.entity[1].securityLabel[0].code), force_bytes("V"))
        self.assertEqual(force_bytes(inst.entity[1].securityLabel[0].display), force_bytes("very restricted"))
        self.assertEqual(force_bytes(inst.entity[1].securityLabel[0].system), force_bytes("http://hl7.org/fhir/v3/Confidentiality"))
        self.assertEqual(force_bytes(inst.entity[1].securityLabel[1].code), force_bytes("STD"))
        self.assertEqual(force_bytes(inst.entity[1].securityLabel[1].display), force_bytes("sexually transmitted disease information sensitivity"))
        self.assertEqual(force_bytes(inst.entity[1].securityLabel[1].system), force_bytes("http://hl7.org/fhir/v3/ActCode"))
        self.assertEqual(force_bytes(inst.entity[1].securityLabel[2].code), force_bytes("DELAU"))
        self.assertEqual(force_bytes(inst.entity[1].securityLabel[2].display), force_bytes("delete after use"))
        self.assertEqual(force_bytes(inst.entity[1].securityLabel[2].system), force_bytes("http://hl7.org/fhir/v3/ActCode"))
        self.assertEqual(force_bytes(inst.entity[1].type.code), force_bytes("2"))
        self.assertEqual(force_bytes(inst.entity[1].type.display), force_bytes("System Object"))
        self.assertEqual(force_bytes(inst.entity[1].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example-disclosure"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("0"))
        self.assertEqual(force_bytes(inst.outcomeDesc), force_bytes("Successful  Disclosure"))
        self.assertEqual(force_bytes(inst.purposeOfEvent[0].coding[0].code), force_bytes("HMARKT"))
        self.assertEqual(force_bytes(inst.purposeOfEvent[0].coding[0].display), force_bytes("healthcare marketing"))
        self.assertEqual(force_bytes(inst.purposeOfEvent[0].coding[0].system), force_bytes("http://hl7.org/fhir/v3/ActReason"))
        self.assertEqual(inst.recorded.date, FHIRDate("2013-09-22T00:08:00Z").date)
        self.assertEqual(inst.recorded.as_json(), "2013-09-22T00:08:00Z")
        self.assertEqual(force_bytes(inst.source.identifier.value), force_bytes("Watchers Accounting of Disclosures Application"))
        self.assertEqual(force_bytes(inst.source.site), force_bytes("Watcher"))
        self.assertEqual(force_bytes(inst.source.type[0].code), force_bytes("4"))
        self.assertEqual(force_bytes(inst.source.type[0].display), force_bytes("Application Server"))
        self.assertEqual(force_bytes(inst.source.type[0].system), force_bytes("http://hl7.org/fhir/security-source-type"))
        self.assertEqual(force_bytes(inst.subtype[0].code), force_bytes("Disclosure"))
        self.assertEqual(force_bytes(inst.subtype[0].display), force_bytes("HIPAA disclosure"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">Disclosure by some idiot, for marketing reasons, to places unknown, of a Poor Sap, data about Everthing important.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.code), force_bytes("110106"))
        self.assertEqual(force_bytes(inst.type.display), force_bytes("Export"))
        self.assertEqual(force_bytes(inst.type.system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
    
    def testAuditEvent3(self):
        inst = self.instantiate_from("audit-event-example-search.json")
        self.assertIsNotNone(inst, "Must have instantiated a AuditEvent instance")
        self.implAuditEvent3(inst)
        
        js = inst.as_json()
        self.assertEqual("AuditEvent", js["resourceType"])
        inst2 = auditevent.AuditEvent(js)
        self.implAuditEvent3(inst2)
    
    def implAuditEvent3(self, inst):
        self.assertEqual(force_bytes(inst.action), force_bytes("E"))
        self.assertEqual(force_bytes(inst.agent[0].altId), force_bytes("601847123"))
        self.assertEqual(force_bytes(inst.agent[0].name), force_bytes("Grahame Grieve"))
        self.assertTrue(inst.agent[0].requestor)
        self.assertEqual(force_bytes(inst.agent[0].userId.value), force_bytes("95"))
        self.assertEqual(force_bytes(inst.agent[1].altId), force_bytes("6580"))
        self.assertEqual(force_bytes(inst.agent[1].network.address), force_bytes("Workstation1.ehr.familyclinic.com"))
        self.assertEqual(force_bytes(inst.agent[1].network.type), force_bytes("1"))
        self.assertFalse(inst.agent[1].requestor)
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].code), force_bytes("110153"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].display), force_bytes("Source Role ID"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[1].userId.value), force_bytes("2.16.840.1.113883.4.2|2.16.840.1.113883.4.2"))
        self.assertEqual(force_bytes(inst.entity[0].query), force_bytes("aHR0cDovL2ZoaXItZGV2LmhlYWx0aGludGVyc2VjdGlvbnMuY29tLmF1L29wZW4vRW5jb3VudGVyP3BhcnRpY2lwYW50PTEz"))
        self.assertEqual(force_bytes(inst.entity[0].role.code), force_bytes("24"))
        self.assertEqual(force_bytes(inst.entity[0].role.display), force_bytes("Query"))
        self.assertEqual(force_bytes(inst.entity[0].role.system), force_bytes("http://hl7.org/fhir/object-role"))
        self.assertEqual(force_bytes(inst.entity[0].type.code), force_bytes("2"))
        self.assertEqual(force_bytes(inst.entity[0].type.display), force_bytes("System Object"))
        self.assertEqual(force_bytes(inst.entity[0].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example-search"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("0"))
        self.assertEqual(inst.recorded.date, FHIRDate("2015-08-22T23:42:24Z").date)
        self.assertEqual(inst.recorded.as_json(), "2015-08-22T23:42:24Z")
        self.assertEqual(force_bytes(inst.source.identifier.value), force_bytes("hl7connect.healthintersections.com.au"))
        self.assertEqual(force_bytes(inst.source.site), force_bytes("Cloud"))
        self.assertEqual(force_bytes(inst.source.type[0].code), force_bytes("3"))
        self.assertEqual(force_bytes(inst.source.type[0].display), force_bytes("Web Server"))
        self.assertEqual(force_bytes(inst.source.type[0].system), force_bytes("http://hl7.org/fhir/security-source-type"))
        self.assertEqual(force_bytes(inst.subtype[0].code), force_bytes("search"))
        self.assertEqual(force_bytes(inst.subtype[0].display), force_bytes("search"))
        self.assertEqual(force_bytes(inst.subtype[0].system), force_bytes("http://hl7.org/fhir/restful-interaction"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.code), force_bytes("rest"))
        self.assertEqual(force_bytes(inst.type.display), force_bytes("Restful Operation"))
        self.assertEqual(force_bytes(inst.type.system), force_bytes("http://hl7.org/fhir/audit-event-type"))
    
    def testAuditEvent4(self):
        inst = self.instantiate_from("audit-event-example-pixQuery.json")
        self.assertIsNotNone(inst, "Must have instantiated a AuditEvent instance")
        self.implAuditEvent4(inst)
        
        js = inst.as_json()
        self.assertEqual("AuditEvent", js["resourceType"])
        inst2 = auditevent.AuditEvent(js)
        self.implAuditEvent4(inst2)
    
    def implAuditEvent4(self, inst):
        self.assertEqual(force_bytes(inst.action), force_bytes("E"))
        self.assertEqual(force_bytes(inst.agent[0].altId), force_bytes("6580"))
        self.assertEqual(force_bytes(inst.agent[0].network.address), force_bytes("Workstation1.ehr.familyclinic.com"))
        self.assertEqual(force_bytes(inst.agent[0].network.type), force_bytes("1"))
        self.assertFalse(inst.agent[0].requestor)
        self.assertEqual(force_bytes(inst.agent[0].role[0].coding[0].code), force_bytes("110153"))
        self.assertEqual(force_bytes(inst.agent[0].role[0].coding[0].display), force_bytes("Source Role ID"))
        self.assertEqual(force_bytes(inst.agent[0].role[0].coding[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[0].userId.value), force_bytes("2.16.840.1.113883.4.2|2.16.840.1.113883.4.2"))
        self.assertEqual(force_bytes(inst.agent[1].altId), force_bytes("601847123"))
        self.assertEqual(force_bytes(inst.agent[1].name), force_bytes("Grahame Grieve"))
        self.assertTrue(inst.agent[1].requestor)
        self.assertEqual(force_bytes(inst.agent[1].userId.value), force_bytes("95"))
        self.assertEqual(force_bytes(inst.entity[0].identifier.value), force_bytes("e3cdfc81a0d24bd^^^&2.16.840.1.113883.4.2&ISO"))
        self.assertEqual(force_bytes(inst.entity[0].role.code), force_bytes("1"))
        self.assertEqual(force_bytes(inst.entity[0].role.display), force_bytes("Patient"))
        self.assertEqual(force_bytes(inst.entity[0].role.system), force_bytes("http://hl7.org/fhir/object-role"))
        self.assertEqual(force_bytes(inst.entity[0].type.code), force_bytes("1"))
        self.assertEqual(force_bytes(inst.entity[0].type.display), force_bytes("Person"))
        self.assertEqual(force_bytes(inst.entity[0].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.entity[1].detail[0].type), force_bytes("MSH-10"))
        self.assertEqual(force_bytes(inst.entity[1].detail[0].value), force_bytes("MS4yLjg0MC4xMTQzNTAuMS4xMy4wLjEuNy4xLjE="))
        self.assertEqual(force_bytes(inst.entity[1].role.code), force_bytes("24"))
        self.assertEqual(force_bytes(inst.entity[1].role.display), force_bytes("Query"))
        self.assertEqual(force_bytes(inst.entity[1].role.system), force_bytes("http://hl7.org/fhir/object-role"))
        self.assertEqual(force_bytes(inst.entity[1].type.code), force_bytes("2"))
        self.assertEqual(force_bytes(inst.entity[1].type.display), force_bytes("System Object"))
        self.assertEqual(force_bytes(inst.entity[1].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example-pixQuery"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("0"))
        self.assertEqual(inst.recorded.date, FHIRDate("2015-08-26T23:42:24Z").date)
        self.assertEqual(inst.recorded.as_json(), "2015-08-26T23:42:24Z")
        self.assertEqual(force_bytes(inst.source.identifier.value), force_bytes("hl7connect.healthintersections.com.au"))
        self.assertEqual(force_bytes(inst.subtype[0].code), force_bytes("ITI-9"))
        self.assertEqual(force_bytes(inst.subtype[0].display), force_bytes("PIX Query"))
        self.assertEqual(force_bytes(inst.subtype[0].system), force_bytes("urn:oid:1.3.6.1.4.1.19376.1.2"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.code), force_bytes("110112"))
        self.assertEqual(force_bytes(inst.type.display), force_bytes("Query"))
        self.assertEqual(force_bytes(inst.type.system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
    
    def testAuditEvent5(self):
        inst = self.instantiate_from("audit-event-example-media.json")
        self.assertIsNotNone(inst, "Must have instantiated a AuditEvent instance")
        self.implAuditEvent5(inst)
        
        js = inst.as_json()
        self.assertEqual("AuditEvent", js["resourceType"])
        inst2 = auditevent.AuditEvent(js)
        self.implAuditEvent5(inst2)
    
    def implAuditEvent5(self, inst):
        self.assertEqual(force_bytes(inst.action), force_bytes("R"))
        self.assertFalse(inst.agent[0].requestor)
        self.assertEqual(force_bytes(inst.agent[0].role[0].coding[0].code), force_bytes("110153"))
        self.assertEqual(force_bytes(inst.agent[0].role[0].coding[0].display), force_bytes("Source Role ID"))
        self.assertEqual(force_bytes(inst.agent[0].role[0].coding[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[0].userId.value), force_bytes("ExportToMedia.app"))
        self.assertEqual(force_bytes(inst.agent[1].altId), force_bytes("601847123"))
        self.assertEqual(force_bytes(inst.agent[1].name), force_bytes("Grahame Grieve"))
        self.assertTrue(inst.agent[1].requestor)
        self.assertEqual(force_bytes(inst.agent[1].userId.value), force_bytes("95"))
        self.assertEqual(force_bytes(inst.agent[2].media.code), force_bytes("110033"))
        self.assertEqual(force_bytes(inst.agent[2].media.display), force_bytes("DVD"))
        self.assertEqual(force_bytes(inst.agent[2].media.system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[2].name), force_bytes("Media title: Hello World"))
        self.assertFalse(inst.agent[2].requestor)
        self.assertEqual(force_bytes(inst.entity[0].identifier.value), force_bytes("e3cdfc81a0d24bd^^^&2.16.840.1.113883.4.2&ISO"))
        self.assertEqual(force_bytes(inst.entity[0].role.code), force_bytes("1"))
        self.assertEqual(force_bytes(inst.entity[0].role.display), force_bytes("Patient"))
        self.assertEqual(force_bytes(inst.entity[0].role.system), force_bytes("http://hl7.org/fhir/object-role"))
        self.assertEqual(force_bytes(inst.entity[0].type.code), force_bytes("1"))
        self.assertEqual(force_bytes(inst.entity[0].type.display), force_bytes("Person"))
        self.assertEqual(force_bytes(inst.entity[0].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.entity[1].identifier.type.coding[0].code), force_bytes("IHE XDS Metadata"))
        self.assertEqual(force_bytes(inst.entity[1].identifier.type.coding[0].display), force_bytes("submission set classificationNode"))
        self.assertEqual(force_bytes(inst.entity[1].identifier.type.coding[0].system), force_bytes("urn:uuid:a54d6aa5-d40d-43f9-88c5-b4633d873bdd"))
        self.assertEqual(force_bytes(inst.entity[1].identifier.value), force_bytes("e3cdfc81a0d24bd^^^&2.16.840.1.113883.4.2&ISO"))
        self.assertEqual(force_bytes(inst.entity[1].role.code), force_bytes("20"))
        self.assertEqual(force_bytes(inst.entity[1].role.display), force_bytes("Job"))
        self.assertEqual(force_bytes(inst.entity[1].role.system), force_bytes("http://hl7.org/fhir/object-role"))
        self.assertEqual(force_bytes(inst.entity[1].type.code), force_bytes("2"))
        self.assertEqual(force_bytes(inst.entity[1].type.display), force_bytes("System Object"))
        self.assertEqual(force_bytes(inst.entity[1].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.entity[2].type.code), force_bytes("2"))
        self.assertEqual(force_bytes(inst.entity[2].type.display), force_bytes("System Object"))
        self.assertEqual(force_bytes(inst.entity[2].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example-media"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("0"))
        self.assertEqual(inst.recorded.date, FHIRDate("2015-08-27T23:42:24Z").date)
        self.assertEqual(inst.recorded.as_json(), "2015-08-27T23:42:24Z")
        self.assertEqual(force_bytes(inst.source.identifier.value), force_bytes("hl7connect.healthintersections.com.au"))
        self.assertEqual(force_bytes(inst.subtype[0].code), force_bytes("ITI-32"))
        self.assertEqual(force_bytes(inst.subtype[0].display), force_bytes("Distribute Document Set on Media"))
        self.assertEqual(force_bytes(inst.subtype[0].system), force_bytes("urn:oid:1.3.6.1.4.1.19376.1.2"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.code), force_bytes("110106"))
        self.assertEqual(force_bytes(inst.type.display), force_bytes("Export"))
        self.assertEqual(force_bytes(inst.type.system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
    
    def testAuditEvent6(self):
        inst = self.instantiate_from("audit-event-example-logout.json")
        self.assertIsNotNone(inst, "Must have instantiated a AuditEvent instance")
        self.implAuditEvent6(inst)
        
        js = inst.as_json()
        self.assertEqual("AuditEvent", js["resourceType"])
        inst2 = auditevent.AuditEvent(js)
        self.implAuditEvent6(inst2)
    
    def implAuditEvent6(self, inst):
        self.assertEqual(force_bytes(inst.action), force_bytes("E"))
        self.assertEqual(force_bytes(inst.agent[0].altId), force_bytes("601847123"))
        self.assertEqual(force_bytes(inst.agent[0].name), force_bytes("Grahame Grieve"))
        self.assertEqual(force_bytes(inst.agent[0].network.address), force_bytes("127.0.0.1"))
        self.assertEqual(force_bytes(inst.agent[0].network.type), force_bytes("2"))
        self.assertTrue(inst.agent[0].requestor)
        self.assertEqual(force_bytes(inst.agent[0].userId.value), force_bytes("95"))
        self.assertEqual(force_bytes(inst.agent[1].altId), force_bytes("6580"))
        self.assertEqual(force_bytes(inst.agent[1].network.address), force_bytes("Workstation1.ehr.familyclinic.com"))
        self.assertEqual(force_bytes(inst.agent[1].network.type), force_bytes("1"))
        self.assertFalse(inst.agent[1].requestor)
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].code), force_bytes("110153"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].display), force_bytes("Source Role ID"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[1].userId.value), force_bytes("2.16.840.1.113883.4.2|2.16.840.1.113883.4.2"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example-logout"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("0"))
        self.assertEqual(inst.recorded.date, FHIRDate("2013-06-20T23:46:41Z").date)
        self.assertEqual(inst.recorded.as_json(), "2013-06-20T23:46:41Z")
        self.assertEqual(force_bytes(inst.source.identifier.value), force_bytes("hl7connect.healthintersections.com.au"))
        self.assertEqual(force_bytes(inst.source.site), force_bytes("Cloud"))
        self.assertEqual(force_bytes(inst.source.type[0].code), force_bytes("3"))
        self.assertEqual(force_bytes(inst.source.type[0].display), force_bytes("Web Server"))
        self.assertEqual(force_bytes(inst.source.type[0].system), force_bytes("http://hl7.org/fhir/security-source-type"))
        self.assertEqual(force_bytes(inst.subtype[0].code), force_bytes("110123"))
        self.assertEqual(force_bytes(inst.subtype[0].display), force_bytes("Logout"))
        self.assertEqual(force_bytes(inst.subtype[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.code), force_bytes("110114"))
        self.assertEqual(force_bytes(inst.type.display), force_bytes("User Authentication"))
        self.assertEqual(force_bytes(inst.type.system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
    
    def testAuditEvent7(self):
        inst = self.instantiate_from("audit-event-example-login.json")
        self.assertIsNotNone(inst, "Must have instantiated a AuditEvent instance")
        self.implAuditEvent7(inst)
        
        js = inst.as_json()
        self.assertEqual("AuditEvent", js["resourceType"])
        inst2 = auditevent.AuditEvent(js)
        self.implAuditEvent7(inst2)
    
    def implAuditEvent7(self, inst):
        self.assertEqual(force_bytes(inst.action), force_bytes("E"))
        self.assertEqual(force_bytes(inst.agent[0].altId), force_bytes("601847123"))
        self.assertEqual(force_bytes(inst.agent[0].name), force_bytes("Grahame Grieve"))
        self.assertEqual(force_bytes(inst.agent[0].network.address), force_bytes("127.0.0.1"))
        self.assertEqual(force_bytes(inst.agent[0].network.type), force_bytes("2"))
        self.assertTrue(inst.agent[0].requestor)
        self.assertEqual(force_bytes(inst.agent[0].userId.value), force_bytes("95"))
        self.assertEqual(force_bytes(inst.agent[1].altId), force_bytes("6580"))
        self.assertEqual(force_bytes(inst.agent[1].network.address), force_bytes("Workstation1.ehr.familyclinic.com"))
        self.assertEqual(force_bytes(inst.agent[1].network.type), force_bytes("1"))
        self.assertFalse(inst.agent[1].requestor)
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].code), force_bytes("110153"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].display), force_bytes("Source Role ID"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[1].userId.value), force_bytes("2.16.840.1.113883.4.2|2.16.840.1.113883.4.2"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example-login"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("0"))
        self.assertEqual(inst.recorded.date, FHIRDate("2013-06-20T23:41:23Z").date)
        self.assertEqual(inst.recorded.as_json(), "2013-06-20T23:41:23Z")
        self.assertEqual(force_bytes(inst.source.identifier.value), force_bytes("hl7connect.healthintersections.com.au"))
        self.assertEqual(force_bytes(inst.source.site), force_bytes("Cloud"))
        self.assertEqual(force_bytes(inst.source.type[0].code), force_bytes("3"))
        self.assertEqual(force_bytes(inst.source.type[0].display), force_bytes("Web Server"))
        self.assertEqual(force_bytes(inst.source.type[0].system), force_bytes("http://hl7.org/fhir/security-source-type"))
        self.assertEqual(force_bytes(inst.subtype[0].code), force_bytes("110122"))
        self.assertEqual(force_bytes(inst.subtype[0].display), force_bytes("Login"))
        self.assertEqual(force_bytes(inst.subtype[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.code), force_bytes("110114"))
        self.assertEqual(force_bytes(inst.type.display), force_bytes("User Authentication"))
        self.assertEqual(force_bytes(inst.type.system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
    
    def testAuditEvent8(self):
        inst = self.instantiate_from("auditevent-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a AuditEvent instance")
        self.implAuditEvent8(inst)
        
        js = inst.as_json()
        self.assertEqual("AuditEvent", js["resourceType"])
        inst2 = auditevent.AuditEvent(js)
        self.implAuditEvent8(inst2)
    
    def implAuditEvent8(self, inst):
        self.assertEqual(force_bytes(inst.action), force_bytes("E"))
        self.assertEqual(force_bytes(inst.agent[0].network.address), force_bytes("127.0.0.1"))
        self.assertEqual(force_bytes(inst.agent[0].network.type), force_bytes("2"))
        self.assertFalse(inst.agent[0].requestor)
        self.assertEqual(force_bytes(inst.agent[0].role[0].text), force_bytes("Service User (Logon)"))
        self.assertEqual(force_bytes(inst.agent[0].userId.value), force_bytes("Grahame"))
        self.assertEqual(force_bytes(inst.agent[1].altId), force_bytes("6580"))
        self.assertEqual(force_bytes(inst.agent[1].network.address), force_bytes("Workstation1.ehr.familyclinic.com"))
        self.assertEqual(force_bytes(inst.agent[1].network.type), force_bytes("1"))
        self.assertFalse(inst.agent[1].requestor)
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].code), force_bytes("110153"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].display), force_bytes("Source Role ID"))
        self.assertEqual(force_bytes(inst.agent[1].role[0].coding[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.agent[1].userId.value), force_bytes("2.16.840.1.113883.4.2|2.16.840.1.113883.4.2"))
        self.assertEqual(force_bytes(inst.entity[0].identifier.type.coding[0].code), force_bytes("SNO"))
        self.assertEqual(force_bytes(inst.entity[0].identifier.type.coding[0].system), force_bytes("http://hl7.org/fhir/identifier-type"))
        self.assertEqual(force_bytes(inst.entity[0].identifier.type.text), force_bytes("Dell Serial Number"))
        self.assertEqual(force_bytes(inst.entity[0].identifier.value), force_bytes("ABCDEF"))
        self.assertEqual(force_bytes(inst.entity[0].lifecycle.code), force_bytes("6"))
        self.assertEqual(force_bytes(inst.entity[0].lifecycle.display), force_bytes("Access / Use"))
        self.assertEqual(force_bytes(inst.entity[0].lifecycle.system), force_bytes("http://hl7.org/fhir/dicom-audit-lifecycle"))
        self.assertEqual(force_bytes(inst.entity[0].name), force_bytes("Grahame's Laptop"))
        self.assertEqual(force_bytes(inst.entity[0].role.code), force_bytes("4"))
        self.assertEqual(force_bytes(inst.entity[0].role.display), force_bytes("Domain Resource"))
        self.assertEqual(force_bytes(inst.entity[0].role.system), force_bytes("http://hl7.org/fhir/object-role"))
        self.assertEqual(force_bytes(inst.entity[0].type.code), force_bytes("4"))
        self.assertEqual(force_bytes(inst.entity[0].type.display), force_bytes("Other"))
        self.assertEqual(force_bytes(inst.entity[0].type.system), force_bytes("http://hl7.org/fhir/object-type"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("0"))
        self.assertEqual(inst.recorded.date, FHIRDate("2012-10-25T22:04:27+11:00").date)
        self.assertEqual(inst.recorded.as_json(), "2012-10-25T22:04:27+11:00")
        self.assertEqual(force_bytes(inst.source.identifier.value), force_bytes("Grahame's Laptop"))
        self.assertEqual(force_bytes(inst.source.site), force_bytes("Development"))
        self.assertEqual(force_bytes(inst.source.type[0].code), force_bytes("110122"))
        self.assertEqual(force_bytes(inst.source.type[0].display), force_bytes("Login"))
        self.assertEqual(force_bytes(inst.source.type[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.subtype[0].code), force_bytes("110120"))
        self.assertEqual(force_bytes(inst.subtype[0].display), force_bytes("Application Start"))
        self.assertEqual(force_bytes(inst.subtype[0].system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">Application Start for under service login &quot;Grahame&quot; (id: Grahame's Test HL7Connect)</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.code), force_bytes("110100"))
        self.assertEqual(force_bytes(inst.type.display), force_bytes("Application Activity"))
        self.assertEqual(force_bytes(inst.type.system), force_bytes("http://dicom.nema.org/resources/ontology/DCM"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import documentmanifest
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class DocumentManifestTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("DocumentManifest", js["resourceType"])
        return documentmanifest.DocumentManifest(js)
    
    def testDocumentManifest1(self):
        inst = self.instantiate_from("documentmanifest-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a DocumentManifest instance")
        self.implDocumentManifest1(inst)
        
        js = inst.as_json()
        self.assertEqual("DocumentManifest", js["resourceType"])
        inst2 = documentmanifest.DocumentManifest(js)
        self.implDocumentManifest1(inst2)
    
    def implDocumentManifest1(self, inst):
        self.assertEqual(force_bytes(inst.contained[0].id), force_bytes("a1"))
        self.assertEqual(inst.created.date, FHIRDate("2004-12-25T23:50:50-05:00").date)
        self.assertEqual(inst.created.as_json(), "2004-12-25T23:50:50-05:00")
        self.assertEqual(force_bytes(inst.description), force_bytes("Physical"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://example.org/documents"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("23425234234-2347"))
        self.assertEqual(force_bytes(inst.masterIdentifier.system), force_bytes("http://example.org/documents"))
        self.assertEqual(force_bytes(inst.masterIdentifier.value), force_bytes("23425234234-2346"))
        self.assertEqual(force_bytes(inst.related[0].identifier.system), force_bytes("http://example.org/documents"))
        self.assertEqual(force_bytes(inst.related[0].identifier.value), force_bytes("23425234234-9999"))
        self.assertEqual(force_bytes(inst.source), force_bytes("urn:oid:1.3.6.1.4.1.21367.2009.1.2.1"))
        self.assertEqual(force_bytes(inst.status), force_bytes("current"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">Text</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.text), force_bytes("History and Physical"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import operationdefinition
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class OperationDefinitionTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("OperationDefinition", js["resourceType"])
        return operationdefinition.OperationDefinition(js)
    
    def testOperationDefinition1(self):
        inst = self.instantiate_from("operationdefinition-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a OperationDefinition instance")
        self.implOperationDefinition1(inst)
        
        js = inst.as_json()
        self.assertEqual("OperationDefinition", js["resourceType"])
        inst2 = operationdefinition.OperationDefinition(js)
        self.implOperationDefinition1(inst2)
    
    def implOperationDefinition1(self, inst):
        self.assertEqual(force_bytes(inst.code), force_bytes("populate"))
        self.assertEqual(force_bytes(inst.comment), force_bytes("Only implemented for Labs and Medications so far"))
        self.assertEqual(force_bytes(inst.contact[0].name), force_bytes("System Administrator"))
        self.assertEqual(force_bytes(inst.contact[0].telecom[0].system), force_bytes("email"))
        self.assertEqual(force_bytes(inst.contact[0].telecom[0].value), force_bytes("beep@coyote.acme.com"))
        self.assertEqual(inst.date.date, FHIRDate("2015-08-04").date)
        self.assertEqual(inst.date.as_json(), "2015-08-04")
        self.assertEqual(force_bytes(inst.description), force_bytes("Limited implementation of the Populate Questionnaire implemenation"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertTrue(inst.instance)
        self.assertEqual(force_bytes(inst.jurisdiction[0].coding[0].code), force_bytes("GB"))
        self.assertEqual(force_bytes(inst.jurisdiction[0].coding[0].display), force_bytes("United Kingdom of Great Britain and Northern Ireland (the)"))
        self.assertEqual(force_bytes(inst.jurisdiction[0].coding[0].system), force_bytes("urn:iso:std:iso:3166"))
        self.assertEqual(force_bytes(inst.kind), force_bytes("operation"))
        self.assertEqual(force_bytes(inst.name), force_bytes("Populate Questionnaire"))
        self.assertEqual(force_bytes(inst.overload[0].parameterName[0]), force_bytes("subject"))
        self.assertEqual(force_bytes(inst.overload[0].parameterName[1]), force_bytes("local"))
        self.assertEqual(force_bytes(inst.overload[1].comment), force_bytes("local defaults to false when not passed as a parameter"))
        self.assertEqual(force_bytes(inst.overload[1].parameterName[0]), force_bytes("subject"))
        self.assertEqual(force_bytes(inst.parameter[0].max), force_bytes("1"))
        self.assertEqual(inst.parameter[0].min, 1)
        self.assertEqual(force_bytes(inst.parameter[0].name), force_bytes("subject"))
        self.assertEqual(force_bytes(inst.parameter[0].type), force_bytes("Reference"))
        self.assertEqual(force_bytes(inst.parameter[0].use), force_bytes("in"))
        self.assertEqual(force_bytes(inst.parameter[1].documentation), force_bytes("If the *local* parameter is set to true, server information about the specified subject will be used to populate the instance."))
        self.assertEqual(force_bytes(inst.parameter[1].max), force_bytes("1"))
        self.assertEqual(inst.parameter[1].min, 0)
        self.assertEqual(force_bytes(inst.parameter[1].name), force_bytes("local"))
        self.assertEqual(force_bytes(inst.parameter[1].type), force_bytes("Reference"))
        self.assertEqual(force_bytes(inst.parameter[1].use), force_bytes("in"))
        self.assertEqual(force_bytes(inst.parameter[2].documentation), force_bytes("The partially (or fully)-populated set of answers for the specified Questionnaire"))
        self.assertEqual(force_bytes(inst.parameter[2].max), force_bytes("1"))
        self.assertEqual(inst.parameter[2].min, 1)
        self.assertEqual(force_bytes(inst.parameter[2].name), force_bytes("return"))
        self.assertEqual(force_bytes(inst.parameter[2].type), force_bytes("QuestionnaireResponse"))
        self.assertEqual(force_bytes(inst.parameter[2].use), force_bytes("out"))
        self.assertEqual(force_bytes(inst.publisher), force_bytes("Acme Healthcare Services"))
        self.assertEqual(force_bytes(inst.resource[0]), force_bytes("Questionnaire"))
        self.assertEqual(force_bytes(inst.status), force_bytes("draft"))
        self.assertFalse(inst.system)
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertFalse(inst.type)
        self.assertEqual(force_bytes(inst.url), force_bytes("http://h7.org/fhir/OperationDefinition/example"))
        self.assertEqual(force_bytes(inst.useContext[0].code.code), force_bytes("venue"))
        self.assertEqual(force_bytes(inst.useContext[0].code.display), force_bytes("Clinical Venue"))
        self.assertEqual(force_bytes(inst.useContext[0].code.system), force_bytes("http://build.fhir.org/codesystem-usage-context-type"))
        self.assertEqual(force_bytes(inst.useContext[0].valueCodeableConcept.coding[0].code), force_bytes("IMP"))
        self.assertEqual(force_bytes(inst.useContext[0].valueCodeableConcept.coding[0].display), force_bytes("inpatient encounter"))
        self.assertEqual(force_bytes(inst.useContext[0].valueCodeableConcept.coding[0].system), force_bytes("http://hl7.org/fhir/v3/ActCode"))
        self.assertEqual(force_bytes(inst.version), force_bytes("B"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/Contributor) on 2019-01-12.
#  2019, SMART Health IT.


from . import element

class Contributor(element.Element):
    """ Contributor information.
    
    A contributor to the content of a knowledge asset, including authors,
    editors, reviewers, and endorsers.
    """
    
    resource_type = "Contributor"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.contact = None
        """ Contact details of the contributor.
        List of `ContactDetail` items (represented as `dict` in JSON). """
        
        self.name = None
        """ Who contributed the content.
        Type `str`. """
        
        self.type = None
        """ author | editor | reviewer | endorser.
        Type `str`. """
        
        super(Contributor, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(Contributor, self).elementProperties()
        js.extend([
            ("contact", "contact", contactdetail.ContactDetail, True, None, False),
            ("name", "name", str, False, None, True),
            ("type", "type", str, False, None, True),
        ])
        return js


import sys
try:
    from . import contactdetail
except ImportError:
    contactdetail = sys.modules[__package__ + '.contactdetail']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/Claim) on 2019-01-12.
#  2019, SMART Health IT.


from . import domainresource

class Claim(domainresource.DomainResource):
    """ Claim, Pre-determination or Pre-authorization.
    
    A provider issued list of services and products provided, or to be
    provided, to a patient which is provided to an insurer for payment
    recovery.
    """
    
    resource_type = "Claim"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.accident = None
        """ Details about an accident.
        Type `ClaimAccident` (represented as `dict` in JSON). """
        
        self.billablePeriod = None
        """ Period for charge submission.
        Type `Period` (represented as `dict` in JSON). """
        
        self.careTeam = None
        """ Members of the care team.
        List of `ClaimCareTeam` items (represented as `dict` in JSON). """
        
        self.created = None
        """ Creation date.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.diagnosis = None
        """ List of Diagnosis.
        List of `ClaimDiagnosis` items (represented as `dict` in JSON). """
        
        self.employmentImpacted = None
        """ Period unable to work.
        Type `Period` (represented as `dict` in JSON). """
        
        self.enterer = None
        """ Author.
        Type `FHIRReference` referencing `Practitioner` (represented as `dict` in JSON). """
        
        self.facility = None
        """ Servicing Facility.
        Type `FHIRReference` referencing `Location` (represented as `dict` in JSON). """
        
        self.fundsReserve = None
        """ Funds requested to be reserved.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.hospitalization = None
        """ Period in hospital.
        Type `Period` (represented as `dict` in JSON). """
        
        self.identifier = None
        """ Claim number.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.information = None
        """ Exceptions, special considerations, the condition, situation, prior
        or concurrent issues.
        List of `ClaimInformation` items (represented as `dict` in JSON). """
        
        self.insurance = None
        """ Insurance or medical plan.
        List of `ClaimInsurance` items (represented as `dict` in JSON). """
        
        self.insurer = None
        """ Target.
        Type `FHIRReference` referencing `Organization` (represented as `dict` in JSON). """
        
        self.item = None
        """ Goods and Services.
        List of `ClaimItem` items (represented as `dict` in JSON). """
        
        self.organization = None
        """ Responsible organization.
        Type `FHIRReference` referencing `Organization` (represented as `dict` in JSON). """
        
        self.originalPrescription = None
        """ Original prescription if superceded by fulfiller.
        Type `FHIRReference` referencing `MedicationRequest` (represented as `dict` in JSON). """
        
        self.patient = None
        """ The subject of the Products and Services.
        Type `FHIRReference` referencing `Patient` (represented as `dict` in JSON). """
        
        self.payee = None
        """ Party to be paid any benefits payable.
        Type `ClaimPayee` (represented as `dict` in JSON). """
        
        self.prescription = None
        """ Prescription authorizing services or products.
        Type `FHIRReference` referencing `MedicationRequest, VisionPrescription` (represented as `dict` in JSON). """
        
        self.priority = None
        """ Desired processing priority.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.procedure = None
        """ Procedures performed.
        List of `ClaimProcedure` items (represented as `dict` in JSON). """
        
        self.provider = None
        """ Responsible provider.
        Type `FHIRReference` referencing `Practitioner` (represented as `dict` in JSON). """
        
        self.referral = None
        """ Treatment Referral.
        Type `FHIRReference` referencing `ReferralRequest` (represented as `dict` in JSON). """
        
        self.related = None
        """ Related Claims which may be revelant to processing this claimn.
        List of `ClaimRelated` items (represented as `dict` in JSON). """
        
        self.status = None
        """ active | cancelled | draft | entered-in-error.
        Type `str`. """
        
        self.subType = None
        """ Finer grained claim type information.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.total = None
        """ Total claim cost.
        Type `Money` (represented as `dict` in JSON). """
        
        self.type = None
        """ Type or discipline.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.use = None
        """ complete | proposed | exploratory | other.
        Type `str`. """
        
        super(Claim, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(Claim, self).elementProperties()
        js.extend([
            ("accident", "accident", ClaimAccident, False, None, False),
            ("billablePeriod", "billablePeriod", period.Period, False, None, False),
            ("careTeam", "careTeam", ClaimCareTeam, True, None, False),
            ("created", "created", fhirdate.FHIRDate, False, None, False),
            ("diagnosis", "diagnosis", ClaimDiagnosis, True, None, False),
            ("employmentImpacted", "employmentImpacted", period.Period, False, None, False),
            ("enterer", "enterer", fhirreference.FHIRReference, False, None, False),
            ("facility", "facility", fhirreference.FHIRReference, False, None, False),
            ("fundsReserve", "fundsReserve", codeableconcept.CodeableConcept, False, None, False),
            ("hospitalization", "hospitalization", period.Period, False, None, False),
            ("identifier", "identifier", identifier.Identifier, True, None, False),
            ("information", "information", ClaimInformation, True, None, False),
            ("insurance", "insurance", ClaimInsurance, True, None, False),
            ("insurer", "insurer", fhirreference.FHIRReference, False, None, False),
            ("item", "item", ClaimItem, True, None, False),
            ("organization", "organization", fhirreference.FHIRReference, False, None, False),
            ("originalPrescription", "originalPrescription", fhirreference.FHIRReference, False, None, False),
            ("patient", "patient", fhirreference.FHIRReference, False, None, False),
            ("payee", "payee", ClaimPayee, False, None, False),
            ("prescription", "prescription", fhirreference.FHIRReference, False, None, False),
            ("priority", "priority", codeableconcept.CodeableConcept, False, None, False),
            ("procedure", "procedure", ClaimProcedure, True, None, False),
            ("provider", "provider", fhirreference.FHIRReference, False, None, False),
            ("referral", "referral", fhirreference.FHIRReference, False, None, False),
            ("related", "related", ClaimRelated, True, None, False),
            ("status", "status", str, False, None, False),
            ("subType", "subType", codeableconcept.CodeableConcept, True, None, False),
            ("total", "total", money.Money, False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, False, None, False),
            ("use", "use", str, False, None, False),
        ])
        return js


from . import backboneelement

class ClaimAccident(backboneelement.BackboneElement):
    """ Details about an accident.
    
    An accident which resulted in the need for healthcare services.
    """
    
    resource_type = "ClaimAccident"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.date = None
        """ When the accident occurred
        see information codes
        see information codes.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.locationAddress = None
        """ Accident Place.
        Type `Address` (represented as `dict` in JSON). """
        
        self.locationReference = None
        """ Accident Place.
        Type `FHIRReference` referencing `Location` (represented as `dict` in JSON). """
        
        self.type = None
        """ The nature of the accident.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        super(ClaimAccident, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimAccident, self).elementProperties()
        js.extend([
            ("date", "date", fhirdate.FHIRDate, False, None, True),
            ("locationAddress", "locationAddress", address.Address, False, "location", False),
            ("locationReference", "locationReference", fhirreference.FHIRReference, False, "location", False),
            ("type", "type", codeableconcept.CodeableConcept, False, None, False),
        ])
        return js


class ClaimCareTeam(backboneelement.BackboneElement):
    """ Members of the care team.
    
    The members of the team who provided the overall service as well as their
    role and whether responsible and qualifications.
    """
    
    resource_type = "ClaimCareTeam"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.provider = None
        """ Provider individual or organization.
        Type `FHIRReference` referencing `Practitioner, Organization` (represented as `dict` in JSON). """
        
        self.qualification = None
        """ Type, classification or Specialization.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.responsible = None
        """ Billing provider.
        Type `bool`. """
        
        self.role = None
        """ Role on the team.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.sequence = None
        """ Number to covey order of careTeam.
        Type `int`. """
        
        super(ClaimCareTeam, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimCareTeam, self).elementProperties()
        js.extend([
            ("provider", "provider", fhirreference.FHIRReference, False, None, True),
            ("qualification", "qualification", codeableconcept.CodeableConcept, False, None, False),
            ("responsible", "responsible", bool, False, None, False),
            ("role", "role", codeableconcept.CodeableConcept, False, None, False),
            ("sequence", "sequence", int, False, None, True),
        ])
        return js


class ClaimDiagnosis(backboneelement.BackboneElement):
    """ List of Diagnosis.
    
    List of patient diagnosis for which care is sought.
    """
    
    resource_type = "ClaimDiagnosis"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.diagnosisCodeableConcept = None
        """ Patient's diagnosis.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.diagnosisReference = None
        """ Patient's diagnosis.
        Type `FHIRReference` referencing `Condition` (represented as `dict` in JSON). """
        
        self.packageCode = None
        """ Package billing code.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.sequence = None
        """ Number to covey order of diagnosis.
        Type `int`. """
        
        self.type = None
        """ Timing or nature of the diagnosis.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        super(ClaimDiagnosis, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimDiagnosis, self).elementProperties()
        js.extend([
            ("diagnosisCodeableConcept", "diagnosisCodeableConcept", codeableconcept.CodeableConcept, False, "diagnosis", True),
            ("diagnosisReference", "diagnosisReference", fhirreference.FHIRReference, False, "diagnosis", True),
            ("packageCode", "packageCode", codeableconcept.CodeableConcept, False, None, False),
            ("sequence", "sequence", int, False, None, True),
            ("type", "type", codeableconcept.CodeableConcept, True, None, False),
        ])
        return js


class ClaimInformation(backboneelement.BackboneElement):
    """ Exceptions, special considerations, the condition, situation, prior or
    concurrent issues.
    
    Additional information codes regarding exceptions, special considerations,
    the condition, situation, prior or concurrent issues. Often there are
    mutiple jurisdiction specific valuesets which are required.
    """
    
    resource_type = "ClaimInformation"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.category = None
        """ General class of information.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.code = None
        """ Type of information.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.reason = None
        """ Reason associated with the information.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.sequence = None
        """ Information instance identifier.
        Type `int`. """
        
        self.timingDate = None
        """ When it occurred.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.timingPeriod = None
        """ When it occurred.
        Type `Period` (represented as `dict` in JSON). """
        
        self.valueAttachment = None
        """ Additional Data or supporting information.
        Type `Attachment` (represented as `dict` in JSON). """
        
        self.valueQuantity = None
        """ Additional Data or supporting information.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.valueReference = None
        """ Additional Data or supporting information.
        Type `FHIRReference` referencing `Resource` (represented as `dict` in JSON). """
        
        self.valueString = None
        """ Additional Data or supporting information.
        Type `str`. """
        
        super(ClaimInformation, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimInformation, self).elementProperties()
        js.extend([
            ("category", "category", codeableconcept.CodeableConcept, False, None, True),
            ("code", "code", codeableconcept.CodeableConcept, False, None, False),
            ("reason", "reason", codeableconcept.CodeableConcept, False, None, False),
            ("sequence", "sequence", int, False, None, True),
            ("timingDate", "timingDate", fhirdate.FHIRDate, False, "timing", False),
            ("timingPeriod", "timingPeriod", period.Period, False, "timing", False),
            ("valueAttachment", "valueAttachment", attachment.Attachment, False, "value", False),
            ("valueQuantity", "valueQuantity", quantity.Quantity, False, "value", False),
            ("valueReference", "valueReference", fhirreference.FHIRReference, False, "value", False),
            ("valueString", "valueString", str, False, "value", False),
        ])
        return js


class ClaimInsurance(backboneelement.BackboneElement):
    """ Insurance or medical plan.
    
    Financial instrument by which payment information for health care.
    """
    
    resource_type = "ClaimInsurance"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.businessArrangement = None
        """ Business agreement.
        Type `str`. """
        
        self.claimResponse = None
        """ Adjudication results.
        Type `FHIRReference` referencing `ClaimResponse` (represented as `dict` in JSON). """
        
        self.coverage = None
        """ Insurance information.
        Type `FHIRReference` referencing `Coverage` (represented as `dict` in JSON). """
        
        self.focal = None
        """ Is the focal Coverage.
        Type `bool`. """
        
        self.preAuthRef = None
        """ Pre-Authorization/Determination Reference.
        List of `str` items. """
        
        self.sequence = None
        """ Service instance identifier.
        Type `int`. """
        
        super(ClaimInsurance, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimInsurance, self).elementProperties()
        js.extend([
            ("businessArrangement", "businessArrangement", str, False, None, False),
            ("claimResponse", "claimResponse", fhirreference.FHIRReference, False, None, False),
            ("coverage", "coverage", fhirreference.FHIRReference, False, None, True),
            ("focal", "focal", bool, False, None, True),
            ("preAuthRef", "preAuthRef", str, True, None, False),
            ("sequence", "sequence", int, False, None, True),
        ])
        return js


class ClaimItem(backboneelement.BackboneElement):
    """ Goods and Services.
    
    First tier of goods and services.
    """
    
    resource_type = "ClaimItem"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.bodySite = None
        """ Service Location.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.careTeamLinkId = None
        """ Applicable careTeam members.
        List of `int` items. """
        
        self.category = None
        """ Type of service or product.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.detail = None
        """ Additional items.
        List of `ClaimItemDetail` items (represented as `dict` in JSON). """
        
        self.diagnosisLinkId = None
        """ Applicable diagnoses.
        List of `int` items. """
        
        self.encounter = None
        """ Encounters related to this billed item.
        List of `FHIRReference` items referencing `Encounter` (represented as `dict` in JSON). """
        
        self.factor = None
        """ Price scaling factor.
        Type `float`. """
        
        self.informationLinkId = None
        """ Applicable exception and supporting information.
        List of `int` items. """
        
        self.locationAddress = None
        """ Place of service.
        Type `Address` (represented as `dict` in JSON). """
        
        self.locationCodeableConcept = None
        """ Place of service.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.locationReference = None
        """ Place of service.
        Type `FHIRReference` referencing `Location` (represented as `dict` in JSON). """
        
        self.modifier = None
        """ Service/Product billing modifiers.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.net = None
        """ Total item cost.
        Type `Money` (represented as `dict` in JSON). """
        
        self.procedureLinkId = None
        """ Applicable procedures.
        List of `int` items. """
        
        self.programCode = None
        """ Program specific reason for item inclusion.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.quantity = None
        """ Count of Products or Services.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.revenue = None
        """ Revenue or cost center code.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.sequence = None
        """ Service instance.
        Type `int`. """
        
        self.service = None
        """ Billing Code.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.servicedDate = None
        """ Date or dates of Service.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.servicedPeriod = None
        """ Date or dates of Service.
        Type `Period` (represented as `dict` in JSON). """
        
        self.subSite = None
        """ Service Sub-location.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.udi = None
        """ Unique Device Identifier.
        List of `FHIRReference` items referencing `Device` (represented as `dict` in JSON). """
        
        self.unitPrice = None
        """ Fee, charge or cost per point.
        Type `Money` (represented as `dict` in JSON). """
        
        super(ClaimItem, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimItem, self).elementProperties()
        js.extend([
            ("bodySite", "bodySite", codeableconcept.CodeableConcept, False, None, False),
            ("careTeamLinkId", "careTeamLinkId", int, True, None, False),
            ("category", "category", codeableconcept.CodeableConcept, False, None, False),
            ("detail", "detail", ClaimItemDetail, True, None, False),
            ("diagnosisLinkId", "diagnosisLinkId", int, True, None, False),
            ("encounter", "encounter", fhirreference.FHIRReference, True, None, False),
            ("factor", "factor", float, False, None, False),
            ("informationLinkId", "informationLinkId", int, True, None, False),
            ("locationAddress", "locationAddress", address.Address, False, "location", False),
            ("locationCodeableConcept", "locationCodeableConcept", codeableconcept.CodeableConcept, False, "location", False),
            ("locationReference", "locationReference", fhirreference.FHIRReference, False, "location", False),
            ("modifier", "modifier", codeableconcept.CodeableConcept, True, None, False),
            ("net", "net", money.Money, False, None, False),
            ("procedureLinkId", "procedureLinkId", int, True, None, False),
            ("programCode", "programCode", codeableconcept.CodeableConcept, True, None, False),
            ("quantity", "quantity", quantity.Quantity, False, None, False),
            ("revenue", "revenue", codeableconcept.CodeableConcept, False, None, False),
            ("sequence", "sequence", int, False, None, True),
            ("service", "service", codeableconcept.CodeableConcept, False, None, False),
            ("servicedDate", "servicedDate", fhirdate.FHIRDate, False, "serviced", False),
            ("servicedPeriod", "servicedPeriod", period.Period, False, "serviced", False),
            ("subSite", "subSite", codeableconcept.CodeableConcept, True, None, False),
            ("udi", "udi", fhirreference.FHIRReference, True, None, False),
            ("unitPrice", "unitPrice", money.Money, False, None, False),
        ])
        return js


class ClaimItemDetail(backboneelement.BackboneElement):
    """ Additional items.
    
    Second tier of goods and services.
    """
    
    resource_type = "ClaimItemDetail"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.category = None
        """ Type of service or product.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.factor = None
        """ Price scaling factor.
        Type `float`. """
        
        self.modifier = None
        """ Service/Product billing modifiers.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.net = None
        """ Total additional item cost.
        Type `Money` (represented as `dict` in JSON). """
        
        self.programCode = None
        """ Program specific reason for item inclusion.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.quantity = None
        """ Count of Products or Services.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.revenue = None
        """ Revenue or cost center code.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.sequence = None
        """ Service instance.
        Type `int`. """
        
        self.service = None
        """ Billing Code.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.subDetail = None
        """ Additional items.
        List of `ClaimItemDetailSubDetail` items (represented as `dict` in JSON). """
        
        self.udi = None
        """ Unique Device Identifier.
        List of `FHIRReference` items referencing `Device` (represented as `dict` in JSON). """
        
        self.unitPrice = None
        """ Fee, charge or cost per point.
        Type `Money` (represented as `dict` in JSON). """
        
        super(ClaimItemDetail, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimItemDetail, self).elementProperties()
        js.extend([
            ("category", "category", codeableconcept.CodeableConcept, False, None, False),
            ("factor", "factor", float, False, None, False),
            ("modifier", "modifier", codeableconcept.CodeableConcept, True, None, False),
            ("net", "net", money.Money, False, None, False),
            ("programCode", "programCode", codeableconcept.CodeableConcept, True, None, False),
            ("quantity", "quantity", quantity.Quantity, False, None, False),
            ("revenue", "revenue", codeableconcept.CodeableConcept, False, None, False),
            ("sequence", "sequence", int, False, None, True),
            ("service", "service", codeableconcept.CodeableConcept, False, None, False),
            ("subDetail", "subDetail", ClaimItemDetailSubDetail, True, None, False),
            ("udi", "udi", fhirreference.FHIRReference, True, None, False),
            ("unitPrice", "unitPrice", money.Money, False, None, False),
        ])
        return js


class ClaimItemDetailSubDetail(backboneelement.BackboneElement):
    """ Additional items.
    
    Third tier of goods and services.
    """
    
    resource_type = "ClaimItemDetailSubDetail"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.category = None
        """ Type of service or product.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.factor = None
        """ Price scaling factor.
        Type `float`. """
        
        self.modifier = None
        """ Service/Product billing modifiers.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.net = None
        """ Net additional item cost.
        Type `Money` (represented as `dict` in JSON). """
        
        self.programCode = None
        """ Program specific reason for item inclusion.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.quantity = None
        """ Count of Products or Services.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.revenue = None
        """ Revenue or cost center code.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.sequence = None
        """ Service instance.
        Type `int`. """
        
        self.service = None
        """ Billing Code.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.udi = None
        """ Unique Device Identifier.
        List of `FHIRReference` items referencing `Device` (represented as `dict` in JSON). """
        
        self.unitPrice = None
        """ Fee, charge or cost per point.
        Type `Money` (represented as `dict` in JSON). """
        
        super(ClaimItemDetailSubDetail, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimItemDetailSubDetail, self).elementProperties()
        js.extend([
            ("category", "category", codeableconcept.CodeableConcept, False, None, False),
            ("factor", "factor", float, False, None, False),
            ("modifier", "modifier", codeableconcept.CodeableConcept, True, None, False),
            ("net", "net", money.Money, False, None, False),
            ("programCode", "programCode", codeableconcept.CodeableConcept, True, None, False),
            ("quantity", "quantity", quantity.Quantity, False, None, False),
            ("revenue", "revenue", codeableconcept.CodeableConcept, False, None, False),
            ("sequence", "sequence", int, False, None, True),
            ("service", "service", codeableconcept.CodeableConcept, False, None, False),
            ("udi", "udi", fhirreference.FHIRReference, True, None, False),
            ("unitPrice", "unitPrice", money.Money, False, None, False),
        ])
        return js


class ClaimPayee(backboneelement.BackboneElement):
    """ Party to be paid any benefits payable.
    
    The party to be reimbursed for the services.
    """
    
    resource_type = "ClaimPayee"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.party = None
        """ Party to receive the payable.
        Type `FHIRReference` referencing `Practitioner, Organization, Patient, RelatedPerson` (represented as `dict` in JSON). """
        
        self.resourceType = None
        """ organization | patient | practitioner | relatedperson.
        Type `Coding` (represented as `dict` in JSON). """
        
        self.type = None
        """ Type of party: Subscriber, Provider, other.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        super(ClaimPayee, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimPayee, self).elementProperties()
        js.extend([
            ("party", "party", fhirreference.FHIRReference, False, None, False),
            ("resourceType", "resourceType", coding.Coding, False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, False, None, True),
        ])
        return js


class ClaimProcedure(backboneelement.BackboneElement):
    """ Procedures performed.
    
    Ordered list of patient procedures performed to support the adjudication.
    """
    
    resource_type = "ClaimProcedure"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.date = None
        """ When the procedure was performed.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.procedureCodeableConcept = None
        """ Patient's list of procedures performed.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.procedureReference = None
        """ Patient's list of procedures performed.
        Type `FHIRReference` referencing `Procedure` (represented as `dict` in JSON). """
        
        self.sequence = None
        """ Procedure sequence for reference.
        Type `int`. """
        
        super(ClaimProcedure, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimProcedure, self).elementProperties()
        js.extend([
            ("date", "date", fhirdate.FHIRDate, False, None, False),
            ("procedureCodeableConcept", "procedureCodeableConcept", codeableconcept.CodeableConcept, False, "procedure", True),
            ("procedureReference", "procedureReference", fhirreference.FHIRReference, False, "procedure", True),
            ("sequence", "sequence", int, False, None, True),
        ])
        return js


class ClaimRelated(backboneelement.BackboneElement):
    """ Related Claims which may be revelant to processing this claimn.
    
    Other claims which are related to this claim such as prior claim versions
    or for related services.
    """
    
    resource_type = "ClaimRelated"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.claim = None
        """ Reference to the related claim.
        Type `FHIRReference` referencing `Claim` (represented as `dict` in JSON). """
        
        self.reference = None
        """ Related file or case reference.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.relationship = None
        """ How the reference claim is related.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        super(ClaimRelated, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(ClaimRelated, self).elementProperties()
        js.extend([
            ("claim", "claim", fhirreference.FHIRReference, False, None, False),
            ("reference", "reference", identifier.Identifier, False, None, False),
            ("relationship", "relationship", codeableconcept.CodeableConcept, False, None, False),
        ])
        return js


import sys
try:
    from . import address
except ImportError:
    address = sys.modules[__package__ + '.address']
try:
    from . import attachment
except ImportError:
    attachment = sys.modules[__package__ + '.attachment']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import coding
except ImportError:
    coding = sys.modules[__package__ + '.coding']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import money
except ImportError:
    money = sys.modules[__package__ + '.money']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/Binary) on 2019-01-12.
#  2019, SMART Health IT.


from . import resource

class Binary(resource.Resource):
    """ Pure binary content defined by a format other than FHIR.
    
    A binary resource can contain any content, whether text, image, pdf, zip
    archive, etc.
    """
    
    resource_type = "Binary"
    
    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.content = None
        """ The actual content.
        Type `str`. """
        
        self.contentType = None
        """ MimeType of the binary content.
        Type `str`. """
        
        self.securityContext = None
        """ Access Control Management.
        Type `FHIRReference` referencing `Resource` (represented as `dict` in JSON). """
        
        super(Binary, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(Binary, self).elementProperties()
        js.extend([
            ("content", "content", str, False, None, True),
            ("contentType", "contentType", str, False, None, True),
            ("securityContext", "securityContext", fhirreference.FHIRReference, False, None, False),
        ])
        return js


import sys
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-01-12.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json
import six
from fhir.resources import documentreference
from fhir.resources.fhirdate import FHIRDate

from .fixtures import force_bytes


@pytest.mark.usefixtures("base_settings")
class DocumentReferenceTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("DocumentReference", js["resourceType"])
        return documentreference.DocumentReference(js)
    
    def testDocumentReference1(self):
        inst = self.instantiate_from("documentreference-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a DocumentReference instance")
        self.implDocumentReference1(inst)
        
        js = inst.as_json()
        self.assertEqual("DocumentReference", js["resourceType"])
        inst2 = documentreference.DocumentReference(js)
        self.implDocumentReference1(inst2)
    
    def implDocumentReference1(self, inst):
        self.assertEqual(force_bytes(inst.class_fhir.coding[0].code), force_bytes("History and Physical"))
        self.assertEqual(force_bytes(inst.class_fhir.coding[0].display), force_bytes("History and Physical"))
        self.assertEqual(force_bytes(inst.class_fhir.coding[0].system), force_bytes("http://ihe.net/xds/connectathon/classCodes"))
        self.assertEqual(force_bytes(inst.contained[0].id), force_bytes("a2"))
        self.assertEqual(force_bytes(inst.content[0].attachment.contentType), force_bytes("application/hl7-v3+xml"))
        self.assertEqual(force_bytes(inst.content[0].attachment.hash), force_bytes("2jmj7l5rSw0yVb/vlWAYkK/YBwk="))
        self.assertEqual(force_bytes(inst.content[0].attachment.language), force_bytes("en-US"))
        self.assertEqual(inst.content[0].attachment.size, 3654)
        self.assertEqual(force_bytes(inst.content[0].attachment.url), force_bytes("http://example.org/xds/mhd/Binary/07a6483f-732b-461e-86b6-edb665c45510"))
        self.assertEqual(force_bytes(inst.content[0].format.code), force_bytes("urn:ihe:pcc:handp:2008"))
        self.assertEqual(force_bytes(inst.content[0].format.display), force_bytes("History and Physical Specification"))
        self.assertEqual(force_bytes(inst.content[0].format.system), force_bytes("urn:oid:1.3.6.1.4.1.19376.1.2.3"))
        self.assertEqual(force_bytes(inst.context.event[0].coding[0].code), force_bytes("T-D8200"))
        self.assertEqual(force_bytes(inst.context.event[0].coding[0].display), force_bytes("Arm"))
        self.assertEqual(force_bytes(inst.context.event[0].coding[0].system), force_bytes("http://ihe.net/xds/connectathon/eventCodes"))
        self.assertEqual(force_bytes(inst.context.facilityType.coding[0].code), force_bytes("Outpatient"))
        self.assertEqual(force_bytes(inst.context.facilityType.coding[0].display), force_bytes("Outpatient"))
        self.assertEqual(force_bytes(inst.context.facilityType.coding[0].system), force_bytes("http://www.ihe.net/xds/connectathon/healthcareFacilityTypeCodes"))
        self.assertEqual(inst.context.period.end.date, FHIRDate("2004-12-23T08:01:00+11:00").date)
        self.assertEqual(inst.context.period.end.as_json(), "2004-12-23T08:01:00+11:00")
        self.assertEqual(inst.context.period.start.date, FHIRDate("2004-12-23T08:00:00+11:00").date)
        self.assertEqual(inst.context.period.start.as_json(), "2004-12-23T08:00:00+11:00")
        self.assertEqual(force_bytes(inst.context.practiceSetting.coding[0].code), force_bytes("General Medicine"))
        self.assertEqual(force_bytes(inst.context.practiceSetting.coding[0].display), force_bytes("General Medicine"))
        self.assertEqual(force_bytes(inst.context.practiceSetting.coding[0].system), force_bytes("http://www.ihe.net/xds/connectathon/practiceSettingCodes"))
        self.assertEqual(force_bytes(inst.context.related[0].identifier.system), force_bytes("urn:ietf:rfc:3986"))
        self.assertEqual(force_bytes(inst.context.related[0].identifier.value), force_bytes("urn:oid:1.3.6.1.4.1.21367.2005.3.7.2345"))
        self.assertEqual(inst.created.date, FHIRDate("2005-12-24T09:35:00+11:00").date)
        self.assertEqual(inst.created.as_json(), "2005-12-24T09:35:00+11:00")
        self.assertEqual(force_bytes(inst.description), force_bytes("Physical"))
        self.assertEqual(force_bytes(inst.docStatus), force_bytes("preliminary"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("urn:ietf:rfc:3986"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("urn:oid:1.3.6.1.4.1.21367.2005.3.7.1234"))
        self.assertEqual(inst.indexed.date, FHIRDate("2005-12-24T09:43:41+11:00").date)
        self.assertEqual(inst.indexed.as_json(), "2005-12-24T09:43:41+11:00")
        self.assertEqual(force_bytes(inst.masterIdentifier.system), force_bytes("urn:ietf:rfc:3986"))
        self.assertEqual(force_bytes(inst.masterIdentifier.value), force_bytes("urn:oid:1.3.6.1.4.1.21367.2005.3.7"))
        self.assertEqual(force_bytes(inst.relatesTo[0].code), force_bytes("appends"))
        self.assertEqual(force_bytes(inst.securityLabel[0].coding[0].code), force_bytes("V"))
        self.assertEqual(force_bytes(inst.securityLabel[0].coding[0].display), force_bytes("very restricted"))
        self.assertEqual(force_bytes(inst.securityLabel[0].coding[0].system), force_bytes("http://hl7.org/fhir/v3/Confidentiality"))
        self.assertEqual(force_bytes(inst.status), force_bytes("current"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("34108-1"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Outpatient Note"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://loinc.org"))

